unit LeakChecker;

interface

uses Windows, SysUtils;

implementation

var
  AllocCount: Integer;
  OldMemMgr: TMemoryManager;

function NewGetMem(Size: Integer): Pointer;
begin
  if Size <> 0 then Inc(AllocCount);
  Result := OldMemMgr.GetMem(Size);
end;

function NewFreeMem(P: Pointer): Integer;
begin
  if P <> nil then Dec(AllocCount);
  Result := OldMemMgr.FreeMem(P);
end;

function NewReallocMem(P: Pointer; Size: Integer): Pointer;
begin
  if P = nil then
  begin
    if Size <> 0 then Inc(AllocCount);
  end
  else if Size = 0 then Dec(AllocCount);

  Result := OldMemMgr.ReallocMem(P, Size);
end;

const
  NewMemMgr: TMemoryManager = (
  GetMem: NewGetMem;
  FreeMem: NewFreeMem;
  ReallocMem: NewReallocMem);

initialization
  GetMemoryManager(OldMemMgr);
  SetMemoryManager(NewMemMgr);
finalization
  if AllocCount > 0 then
    MessageBox(0, PChar('Leak count: ' + IntToStr(AllocCount)), 'Memory leak error', MB_OK or MB_ICONEXCLAMATION);
end.

